# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.environment import Environment
from aws_embedded_metrics.logger.metrics_context import MetricsContext
from aws_embedded_metrics.config import get_config
from typing import Any, Awaitable, Callable, Dict

Config = get_config()


class MetricsLogger:
    def __init__(
        self,
        resolve_environment: Callable[..., Awaitable[Environment]],
        context: MetricsContext = None,
    ):
        self.resolve_environment = resolve_environment
        self.context: MetricsContext = context or MetricsContext.empty()

    async def flush(self) -> None:
        # resolve the environment and get the sink
        # MOST of the time this will run synchonrously
        # This only runs asynchronously if executing for the
        # first time in a non-lambda environment
        environment = await self.resolve_environment()

        self.__configureContextForEnvironment(environment)
        sink = environment.get_sink()

        # accept and reset the context
        sink.accept(self.context)
        self.context = self.context.create_copy_with_context()

    def __configureContextForEnvironment(self, env: Environment) -> None:
        default_dimensions = {
            # LogGroup name will entirely depend on the environment since there
            # are some cases where the LogGroup cannot be configured (e.g. Lambda)
            "LogGroup": env.get_log_group_name(),
            "ServiceName": Config.service_name or env.get_name(),
            "ServiceType": Config.service_type or env.get_type(),
        }
        self.context.set_default_dimensions(default_dimensions)
        env.configure_context(self.context)

    def set_property(self, key: str, value: Any) -> "MetricsLogger":
        self.context.set_property(key, value)
        return self

    def put_dimensions(self, dimensions: Dict[str, str]) -> "MetricsLogger":
        self.context.put_dimensions(dimensions)
        return self

    def set_dimensions(self, *dimensions: Dict[str, str]) -> "MetricsLogger":
        self.context.set_dimensions(list(dimensions))
        return self

    def set_namespace(self, namespace: str) -> "MetricsLogger":
        self.context.namespace = namespace
        return self

    def put_metric(self, key: str, value: float, unit: str = "None") -> "MetricsLogger":
        self.context.put_metric(key, value, unit)
        return self

    def new(self) -> "MetricsLogger":
        return MetricsLogger(
            self.resolve_environment, self.context.create_copy_with_context()
        )



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from aws_embedded_metrics.config.configuration import Configuration

ENV_VAR_PREFIX = "AWS_EMF"

ENABLE_DEBUG_LOGGING = "ENABLE_DEBUG_LOGGING"
SERVICE_NAME = "SERVICE_NAME"
SERVICE_TYPE = "SERVICE_TYPE"
LOG_GROUP_NAME = "LOG_GROUP_NAME"
LOG_STREAM_NAME = "LOG_STREAM_NAME"
AGENT_ENDPOINT = "AGENT_ENDPOINT"
EC2_METADATA_ENDPOINT = "EC2_METADATA_ENDPOINT"
NAMESPACE = "NAMESPACE"


class EnvironmentConfigurationProvider:
    """
    Loads configuration from environment variables
    """

    def get_configuration(self) -> Configuration:
        return Configuration(
            self.__get_bool_env_var(ENABLE_DEBUG_LOGGING),
            self.__get_env_var(SERVICE_NAME),
            self.__get_env_var(SERVICE_TYPE),
            self.__get_env_var(LOG_GROUP_NAME),
            self.__get_env_var(LOG_STREAM_NAME),
            self.__get_env_var(AGENT_ENDPOINT),
            self.__get_env_var(EC2_METADATA_ENDPOINT),
            self.__get_env_var(NAMESPACE),
        )

    @staticmethod
    def __get_env_var(key: str) -> str:
        value = os.environ.get(f"{ENV_VAR_PREFIX}_{key}")
        if value is None:
            return ""
        return value

    @staticmethod
    def __get_bool_env_var(key: str) -> bool:
        value = os.environ.get(f"{ENV_VAR_PREFIX}_{key}")
        if value is None:
            return False
        return value.lower() == "true"



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.sinks import SocketClient
import logging
import socket
import threading
from urllib.parse import ParseResult

log = logging.getLogger(__name__)

# TODO: use non-blocking sockets or asyncore


class TcpClient(SocketClient):
    def __init__(self, endpoint: ParseResult):
        self._endpoint = endpoint
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._write_lock = threading.Lock()
        self._should_connect = True

    def connect(self) -> "TcpClient":
        try:
            self._sock.connect((self._endpoint.hostname, self._endpoint.port))
            self._should_connect = False
        except socket.timeout as e:
            log.error("Socket timeout durring connect %s" % (e,))
            self._should_connect = True
        except Exception as e:
            log.error("Failed to connect to the socket. %s" % (e,))
            self._should_connect = True
        return self

    def send_message(self, message: bytes) -> None:
        if self._sock._closed or self._should_connect:  # type: ignore
            self.connect()

        with self._write_lock:
            try:
                self._sock.sendall(message)
                log.info("Submitted metrics to agent over TCP.")
            except socket.timeout as e:
                log.error("Socket timeout durring send %s" % (e,))
                self.connect()
            except socket.error as e:
                log.error("Failed to write metrics to the socket due to socket.error. %s" % (e,))
                self.connect()
            except Exception as e:
                log.error("Failed to write metrics to the socket due to exception. %s" % (e,))
                self.connect()



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

DEFAULT_NAMESPACE = "aws-embedded-metrics"
MAX_DIMENSIONS = 9



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.environment import Environment
from aws_embedded_metrics.logger.metrics_context import MetricsContext
from aws_embedded_metrics.sinks import Sink
from aws_embedded_metrics.sinks.lambda_sink import LambdaSink
import os


def get_env(key: str) -> str:
    if key in os.environ:
        return os.environ[key]
    return ""


sink = LambdaSink()


class LambdaEnvironment(Environment):
    async def probe(self) -> bool:
        return len(get_env("AWS_LAMBDA_FUNCTION_NAME")) > 0

    def get_name(self) -> str:
        return self.get_log_group_name()

    def get_type(self) -> str:
        return "AWS::Lambda::Function"

    def get_log_group_name(self) -> str:
        return get_env("AWS_LAMBDA_FUNCTION_NAME")

    def configure_context(self, context: MetricsContext) -> None:
        context.set_property("executionEnvironment", get_env("AWS_EXECUTION_ENV"))
        context.set_property("memorySize", get_env("AWS_LAMBDA_FUNCTION_MEMORY_SIZE"))
        context.set_property("functionVersion", get_env("AWS_LAMBDA_FUNCTION_VERSION"))
        context.set_property("logStreamId", get_env("AWS_LAMBDA_LOG_STREAM_NAME"))
        trace_id = get_env("_X_AMZN_TRACE_ID")

        if len(trace_id) > 0 and "Sampled=1" in trace_id:
            context.set_property("traceId", trace_id)

    def get_sink(self) -> Sink:
        """Create the appropriate sink for this environment."""
        return sink



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from aws_embedded_metrics.environment import Environment
from aws_embedded_metrics.environment.default_environment import DefaultEnvironment
from aws_embedded_metrics.environment.lambda_environment import LambdaEnvironment
from aws_embedded_metrics.environment.ec2_environment import EC2Environment
from typing import Optional

log = logging.getLogger(__name__)

environments = [LambdaEnvironment(), EC2Environment()]


class EnvironmentCache:
    environment: Optional[Environment] = None


async def resolve_environment() -> Environment:
    if EnvironmentCache.environment is not None:
        log.debug("Environment resolved from cache.")
        return EnvironmentCache.environment

    for env_under_test in environments:
        is_environment = False
        try:
            log.info("Testing environment: %s", env_under_test.__class__.__name__)
            is_environment = await env_under_test.probe()
        except Exception as e:
            log.error(
                "Failed to detect enironment: %s", env_under_test.__class__.__name__, e
            )
            pass

        if is_environment:
            log.info("Detected environment: %s", env_under_test.__class__.__name__)
            EnvironmentCache.environment = env_under_test
            return env_under_test

    log.info("No environment was detected. Using default.")
    EnvironmentCache.environment = DefaultEnvironment()
    return EnvironmentCache.environment



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.sinks import SocketClient
import logging
import socket
from urllib.parse import ParseResult

log = logging.getLogger(__name__)


class UdpClient(SocketClient):
    def __init__(self, endpoint: ParseResult):
        self.endpoint = endpoint

    def send_message(self, message: bytes) -> None:
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.sendto(message, (self.endpoint.hostname, self.endpoint.port))
        sock.close()
        log.info("Submitted metrics to agent over UDP.")



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
from aws_embedded_metrics.logger.metrics_context import MetricsContext
from aws_embedded_metrics.sinks import Sink


class Environment(abc.ABC):
    @abc.abstractmethod
    async def probe(self) -> bool:
        """Determines whether or not we are executing in this environment"""

    @abc.abstractmethod
    def get_name(self) -> str:
        """Get the environment name. This will be used to set the ServiceName dimension."""

    @abc.abstractmethod
    def get_type(self) -> str:
        """Get the environment type. This will be used to set the ServiceType dimension."""

    @abc.abstractmethod
    def get_log_group_name(self) -> str:
        """Get log group name. This will be used to set the LogGroup dimension."""

    @abc.abstractmethod
    def configure_context(self, context: MetricsContext) -> None:
        """Configure the context with environment properties."""

    @abc.abstractmethod
    def get_sink(self) -> Sink:
        """Create the appropriate sink for this environment."""



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from aws_embedded_metrics import constants, utils
from aws_embedded_metrics.config import get_config
from aws_embedded_metrics.logger.metric import Metric
from typing import List, Dict, Any


class MetricsContext(object):
    """
    Stores metrics and their associated properties and dimensions.
    """

    def __init__(
        self,
        namespace: str = None,
        properties: Dict[str, Any] = None,
        dimensions: List[Dict[str, str]] = None,
        default_dimensions: Dict[str, str] = None,
    ):

        self.namespace: str = namespace or get_config().namespace or constants.DEFAULT_NAMESPACE
        self.properties: Dict[str, Any] = properties or {}
        self.dimensions: List[Dict[str, str]] = dimensions or []
        self.default_dimensions: Dict[str, str] = default_dimensions or {}
        self.metrics: Dict[str, Metric] = {}
        self.should_use_default_dimensions = True
        self.meta: Dict[str, Any] = {"Timestamp": utils.now()}

    def put_metric(self, key: str, value: float, unit: str = None) -> None:
        """
        Adds a metric measurement to the context.
        Multiple calls using the same key will be stored as an
        array of scalar values.
        ```
        context.put_metric("Latency", 100, "Milliseconds")
        ```
        """
        metric = self.metrics.get(key)
        if metric:
            # TODO: we should log a warning if the unit has been changed
            metric.add_value(value)
        else:
            self.metrics[key] = Metric(value, unit)

    def put_dimensions(self, dimensions: Dict[str, str]) -> None:
        """
        Adds dimensions to the context.
        ```
        context.put_dimensions({ "k1": "v1", "k2": "v2" })
        ```
        """
        if dimensions is None:
            # TODO add ability to define failure strategy
            return

        self.dimensions.append(dimensions)

    def set_dimensions(self, dimensionSets: List[Dict[str, str]]) -> None:
        """
        Overwrite all dimensions.
        ```
        context.set_dimensions([
            { "k1": "v1" },
            { "k1": "v1", "k2": "v2" }])
        ```
        """
        self.should_use_default_dimensions = False
        self.dimensions = dimensionSets

    def set_default_dimensions(self, default_dimensions: Dict) -> None:
        """
        Sets default dimensions for all other dimensions that get added
        to the context.
        If no custom dimensions are specified, the metrics will be emitted
        with the defaults.
        If custom dimensions are specified, they will be prepended with
        the default dimensions.
        """
        self.default_dimensions = default_dimensions

    def set_property(self, key: str, value: Any) -> None:
        self.properties[key] = value

    def get_dimensions(self) -> List[Dict]:
        """
        Returns the current dimensions on the context
        """
        # user has directly called set_dimensions
        if not self.should_use_default_dimensions:
            return self.dimensions

        if not self.__has_default_dimensions():
            return self.dimensions

        if len(self.dimensions) == 0:
            return [self.default_dimensions]

        # we have to merge dimensions on the read path
        # because defaults won't actually get set until the flush
        # method is called. This allows us to not block the user
        # code while we're detecting the environment
        return list(
            map(lambda custom: {**self.default_dimensions, **custom}, self.dimensions)
        )

    def __has_default_dimensions(self) -> bool:
        return self.default_dimensions is not None and len(self.default_dimensions) > 0

    def create_copy_with_context(self) -> "MetricsContext":
        """
        Creates a deep copy of the context excluding metrics.
        """
        new_properties: Dict = {}
        new_properties.update(self.properties)

        # dimensions added with put_dimension will not be copied.
        # the reason for this is so that you can flush the same scope multiple
        # times without stacking new dimensions. Example:
        #
        # @metric_scope
        # def my_func(metrics):
        #  metrics.put_dimensions(...)
        #
        # my_func()
        # my_func()
        new_dimensions: List[Dict] = []

        new_default_dimensions: Dict = {}
        new_default_dimensions.update(self.default_dimensions)

        return MetricsContext(
            self.namespace, new_properties, new_dimensions, new_default_dimensions
        )

    @staticmethod
    def empty() -> "MetricsContext":
        return MetricsContext()



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.logger.metrics_logger_factory import create_metrics_logger
import inspect
import asyncio
from functools import wraps


def metric_scope(fn):  # type: ignore
    logger = create_metrics_logger()

    if asyncio.iscoroutinefunction(fn):

        @wraps(fn)
        async def wrapper(*args, **kwargs):  # type: ignore
            if "metrics" in inspect.signature(fn).parameters:
                kwargs["metrics"] = logger
            try:
                return await fn(*args, **kwargs)
            except Exception as e:
                raise e
            finally:
                await logger.flush()

        return wrapper
    else:

        @wraps(fn)
        def wrapper(*args, **kwargs):  # type: ignore
            if "metrics" in inspect.signature(fn).parameters:
                kwargs["metrics"] = logger
            try:
                return fn(*args, **kwargs)
            except Exception as e:
                raise e
            finally:
                loop = asyncio.get_event_loop()
                loop.run_until_complete(logger.flush())

        return wrapper



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.config import get_config
from aws_embedded_metrics.logger.metrics_context import MetricsContext
from aws_embedded_metrics.sinks import Sink, SocketClient
from aws_embedded_metrics.sinks.udp_client import UdpClient
from aws_embedded_metrics.sinks.tcp_client import TcpClient
from aws_embedded_metrics.serializers import Serializer
from aws_embedded_metrics.serializers.log_serializer import LogSerializer
import logging
from urllib.parse import urlparse, ParseResult

log = logging.getLogger(__name__)
Config = get_config()

DEFAULT_ENDPOINT = urlparse("tcp://0.0.0.0:25888")


def get_endpoint() -> ParseResult:
    if not Config.agent_endpoint:
        return DEFAULT_ENDPOINT
    try:
        parsed_url = urlparse(Config.agent_endpoint)
        if parsed_url is None or parsed_url.hostname is None or parsed_url.port is None:
            return DEFAULT_ENDPOINT
        else:
            return parsed_url
    except Exception:
        log.debug("Failed to parse agent endpoint: %s", Config.agent_endpoint)
        return DEFAULT_ENDPOINT


def get_socket_client(endpoint: ParseResult) -> SocketClient:
    if endpoint.scheme == "udp":
        return UdpClient(endpoint)
    else:
        return TcpClient(endpoint).connect()


class AgentSink(Sink):
    def __init__(
        self,
        log_group_name: str,
        log_steam_name: str = None,
        serializer: Serializer = LogSerializer(),
    ):
        self.log_group_name = log_group_name
        self.log_steam_name = log_steam_name
        self.serializer = serializer
        self.endpoint = get_endpoint()
        self.client = get_socket_client(self.endpoint)

    def accept(self, context: MetricsContext) -> None:
        context.meta["LogGroupName"] = self.log_group_name
        if self.log_steam_name is not None:
            context.meta["LogStreamName"] = self.log_steam_name

        serialized_content = self.serializer.serialize(context) + '\n'
        log.info(
            "Parsed agent endpoint (%s) %s:%s",
            self.endpoint.scheme,
            self.endpoint.hostname,
            self.endpoint.port,
        )
        self.client.send_message(serialized_content.encode('utf-8'))

    @staticmethod
    def name() -> str:
        return "AgentSink"



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.config import get_config
from aws_embedded_metrics.environment import Environment
from aws_embedded_metrics.logger.metrics_context import MetricsContext
from aws_embedded_metrics.sinks.agent_sink import AgentSink
from typing import Any, Dict, Optional, cast

import aiohttp
import logging


log = logging.getLogger(__name__)
Config = get_config()

DEFAULT_EC2_METADATA_ENDPOINT = (
    "http://169.254.169.254/latest/dynamic/instance-identity/document"
)


async def fetch(  # type: ignore
    session: aiohttp.ClientSession, url: str
) -> Dict[str, Any]:
    async with session.get(url, timeout=2) as response:
        # content_type=None prevents validation of the HTTP Content-Type header
        # The EC2 metadata endpoint uses text/plain instead of application/json
        # https://github.com/aio-libs/aiohttp/blob/7f777333a4ec0043ddf2e8d67146a626089773d9/aiohttp/web_request.py#L582-L585
        return cast(Dict[str, Any], await response.json(content_type=None))


class EC2Environment(Environment):
    def __init__(self) -> None:
        self.sink: Optional[AgentSink] = None

    async def probe(self) -> bool:
        async with aiohttp.ClientSession() as session:
            metadata_endpoint = (
                Config.ec2_metadata_endpoint or DEFAULT_EC2_METADATA_ENDPOINT
            )
            log.info("Fetching EC2 metadata from: %s", metadata_endpoint)
            try:
                response_json = await fetch(session, metadata_endpoint)
                log.debug("Received response from EC2 metdata endpoint.")
                self.metadata = response_json
                return True
            except Exception:
                log.info(
                    "Failed to connect to EC2 metadata endpoint %s", metadata_endpoint
                )
            return False

    def get_name(self) -> str:
        return Config.service_name or "Unknown"

    def get_type(self) -> str:
        if self.metadata is not None:
            return "AWS::EC2::Instance"

        return "Unknown"

    def get_log_group_name(self) -> str:
        return Config.log_group_name or f"{self.get_name()}-metrics"

    def configure_context(self, context: MetricsContext) -> None:
        if self.metadata is not None:
            context.set_property("imageId", self.metadata["imageId"])
            context.set_property("instanceId", self.metadata["instanceId"])
            context.set_property("instanceType", self.metadata["instanceType"])
            context.set_property("privateIp", self.metadata["privateIp"])
            context.set_property("availabilityZone", self.metadata["availabilityZone"])

    def get_sink(self) -> AgentSink:
        if self.sink is None:
            self.sink = AgentSink(self.get_log_group_name(), Config.log_stream_name)
        return self.sink



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
from aws_embedded_metrics.logger.metrics_context import MetricsContext


class Sink(abc.ABC):
    """The mechanism by which logs are sent to their destination."""

    @staticmethod
    @abc.abstractmethod
    def name() -> str:
        """The name of the sink."""

    @abc.abstractmethod
    def accept(self, context: MetricsContext) -> None:
        """Flushes the metrics context to the sink."""


class SocketClient(abc.ABC):
    """Interface for pushing data to a socket"""

    @abc.abstractmethod
    def send_message(self, message: bytes) -> None:
        """Send binary payload to socket"""



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.config.configuration import Configuration
from aws_embedded_metrics.config.environment_configuration_provider import (
    EnvironmentConfigurationProvider,
)

config = EnvironmentConfigurationProvider().get_configuration()


def get_config() -> Configuration:
    """Gets the current configuration"""
    return config



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class Configuration:
    def __init__(
        self,
        debug_logging_enabled: bool,
        service_name: str,
        service_type: str,
        log_group_name: str,
        log_stream_name: str,
        agent_endpoint: str,
        ec2_metadata_endpoint: str = None,
        namespace: str = None,
    ):
        self.debug_logging_enabled = debug_logging_enabled
        self.service_name = service_name
        self.service_type = service_type
        self.log_group_name = log_group_name
        self.log_stream_name = log_stream_name
        self.agent_endpoint = agent_endpoint
        self.ec2_metadata_endpoint = ec2_metadata_endpoint
        self.namespace = namespace



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.config import get_config
from aws_embedded_metrics.environment import Environment
from aws_embedded_metrics.logger.metrics_context import MetricsContext
from aws_embedded_metrics.sinks.agent_sink import AgentSink
from typing import Optional

Config = get_config()


class DefaultEnvironment(Environment):
    def __init__(self) -> None:
        self.sink: Optional[AgentSink] = None

    async def probe(self) -> bool:
        return True

    def get_name(self) -> str:
        return Config.service_name or "Unknown"

    def get_type(self) -> str:
        return Config.service_type or "Unknown"

    def get_log_group_name(self) -> str:
        return Config.log_group_name or f"{self.get_name()}-metrics"

    def configure_context(self, context: MetricsContext) -> None:
        pass

    def get_sink(self) -> AgentSink:
        if self.sink is None:
            self.sink = AgentSink(self.get_log_group_name(), Config.log_stream_name)
        return self.sink



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="aws-embedded-metrics",
    version="1.0.1b4",
    author="Amazon Web Services",
    author_email="jarnance@amazon.com",
    description="AWS Embedded Metrics Package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="aws logs metrics emf",
    url="https://github.com/awslabs/aws-embedded-metrics-python",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: Apache Software License",
    ],
    packages=find_packages(exclude=["tests*"]),
    include_package_data=True,
    install_requires=["aiohttp"],
    test_suite="tests"
)



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.logger.metrics_context import MetricsContext
from aws_embedded_metrics.serializers import Serializer
from aws_embedded_metrics.constants import MAX_DIMENSIONS
import json
from typing import Any, Dict, List


class LogSerializer(Serializer):
    @staticmethod
    def serialize(context: MetricsContext) -> str:
        dimension_keys = []
        dimensions_properties: Dict[str, str] = {}

        for dimension_set in context.get_dimensions():
            keys = list(dimension_set.keys())
            dimension_keys.append(keys[0:MAX_DIMENSIONS])
            dimensions_properties = {**dimensions_properties, **dimension_set}

        metric_pointers: List[Dict[str, str]] = []

        metric_definitions = {
            "Dimensions": dimension_keys,
            "Metrics": metric_pointers,
            "Namespace": context.namespace,
        }
        cloud_watch_metrics = [metric_definitions]

        body: Dict[str, Any] = {
            **dimensions_properties,
            **context.properties,
            "_aws": {**context.meta, "CloudWatchMetrics": cloud_watch_metrics},
        }

        for metric_name, metric in context.metrics.items():

            if len(metric.values) == 1:
                body[metric_name] = metric.values[0]
            else:
                body[metric_name] = metric.values

            metric_pointers.append({"Name": metric_name, "Unit": metric.unit})

        return json.dumps(body)



# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.logger.metrics_context import MetricsContext
from aws_embedded_metrics.sinks import Sink
from aws_embedded_metrics.serializers import Serializer
from aws_embedded_metrics.serializers.log_serializer import LogSerializer


class LambdaSink(Sink):
    def __init__(self, serializer: Serializer = LogSerializer()):
        self.serializer = serializer

    def accept(self, context: MetricsContext) -> None:
        print(self.serializer.serialize(context))

    @staticmethod
    def name() -> str:
        return "LambdaSink"
